import argparse
import signal
import sys, os
import json
import numpy as np
import time
import getpass
from influxdb import InfluxDBClient, exceptions


try:
    import labRemote
except:
    print("Couldn't find the labRemote Python libraries. Make sure to add them to your $PYTHONPATH first by doing")
    print()
    print("     export PYTHONPATH=/path/to/labRemote/build/lib:$PYTHONPATH")
    print()
    print("And make sure also that you have previously built labRemote enabling the Python bindings (cmake3 -DUSE_PYTHON=on)")
    exit(1)

def sigint_handler(sig, frame):
    print('[ info ][ ivi ] Received SIGINT. Exiting!')
    sys.exit(0)


signal.signal(signal.SIGINT, sigint_handler)


def meas_voltage(hw, ps, args):
    return ps.measureVoltage()

def power_on(hw, ps, args):
    if args.voltage_level and args.current_protect:
        ps.setVoltageLevel(args.voltage_level)
        ps.setCurrentProtect(args.current_protect)
    ps.turnOn()
    return 0


def power_off(hw, ps, args):
    ps.turnOff()
    return 0


def powerPassiveChannels(config, key, what):
    
    if what == "turnOn":
        for elem in config[key]["passive_channels"]:
            if elem != {}:

                ps = hw.getPowerSupplyChannel(elem["name"])

                print()
                print("setting I to %2g A and V to %2g V on channel %s" % (elem["current"], elem["voltage"], elem["name"]))
                ps.setVoltageLevel(elem["voltage"])
                ps.setCurrentLevel(elem["current"])

                ps.turnOn()

    elif what == "turnOff":
        for elem in config[key]["passive_channels"]:
            if elem != {}:
                ps = hw.getPowerSupplyChannel(elem["name"])
                ps.turnOn()


def doivi(config, w, dbSink):

    if w == "iv":
        key = "sensor_iv"
        outputKey = "Sensor_IV"
        xVarName = "voltage"; xKeyOutName = "Voltage"; xKeyOutUnit = "V"
        yVarName = "current"; yKeyOutName = "Current"; yKeyOutUnit = "A"

    elif w == "vi":
        key = "module_vi"
        outputKey = "SLDO_VI"
        xVarName = "current"; xKeyOutName = "Current"; xKeyOutUnit = "A"
        yVarName = "voltage"; yKeyOutName = "Voltage"; yKeyOutUnit = "V"


    ch = config[key]["target_channel"]
    ps = hw.getPowerSupplyChannel(ch["name"])
    if ps == None:
        print("[ error ][ ivi ] change the config and try again!")
        exit(1)

    if w == "iv":
        setXvar = ps.setVoltageLevel
        measXvar = ps.measureVoltage
        measYvar = ps.measureCurrent

    elif w == "vi":
        setXvar =  ps.setCurrentLevel
        measXvar = ps.measureCurrent
        measYvar = ps.measureVoltage


    print("Starting the %s scan..." % w)

    if w == "iv": 
        ps.setCurrentProtect(ch["current_protection"])
        
    elif w == "vi":
        ps.setVoltageLevel(ch["setting_voltage"])
        ps.setVoltageProtect(ch["voltage_protection"]) 


    start = time.time()

    powerPassiveChannels(config, key, "turnOn")


    # output as JSON
    # > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > 
    data = {}

    data['unit'] = {}
    data['unit']['Time'] = 's'
    data['unit'][xKeyOutName] = xKeyOutUnit
    data['unit'][yKeyOutName] = yKeyOutUnit

    data[xKeyOutUnit+"_step"]       = config[key]["target_channel"]["step_size"]
    data["Step_duration"]           = config[key]["target_channel"]["sleeping_time_per_step_s"]
    data["N_measurements_per_step"] = config[key]["target_channel"]["measurements_per_step"]
    data[yKeyOutUnit+"_Compliance"] = config[key]["target_channel"][yVarName+"_protection"]

    data[outputKey] = []
    # < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < 

    # Output to the terminal
    # > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > 
    colWidths = "{0:8}{1:17}{2:20}{3:20}{4:20}{5:20}"
    print(colWidths.format("time", xVarName + "_Level", xVarName + "_SenseMean", xVarName + "_SenseStd", yVarName + "_SenseMean", yVarName + "_SenseStd"))
    # < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < 


    xVar = ch["initial_" + xVarName]
    step = ch["step_size"]
    while xVar * step/abs(step) <= ch["final_" + xVarName]:

        setXvar(xVar)
        ps.turnOn()
        
        time.sleep(ch["sleeping_time_per_step_s"])

        xVarPerStep = []
        yVarPerStep = []

        for j in range(ch["measurements_per_step"]):
            time.sleep(0.1)
            measX = measXvar()
            measY = measYvar()

            xVarPerStep.append(measX)
            yVarPerStep.append(measY)

            if measY > ch[yVarName + "_protection"]:
                print("Reached " + yVarName + " protection value")
                break
        else:

            elapsedTime = time.time() - start

            xVarMean = np.mean(xVarPerStep)
            xVarStd  = np.std(xVarPerStep)
            yVarMean = np.mean(yVarPerStep)
            yVarStd  = np.std(yVarPerStep)

            data[outputKey].append({"Time" : elapsedTime})
            data[outputKey][-1][xKeyOutName] = xVarMean
            data[outputKey][-1][yKeyOutName+"_mean"] = yVarMean
            data[outputKey][-1][yKeyOutName+"_sigma"] = yVarStd

            # Terminal output
            print(colWidths.format("%.2g" % elapsedTime, "%.2g" % xVar, "%.2g" % xVarMean, "%.2g" % xVarStd, "%.2g" % yVarMean, "%.2g" % yVarStd))


            # Output to InfluxDB
            # > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > > 
            point = {
                "measurement": dbSink["measurement"],
                "fields": {
                    "x_" + xVarName + "Setting": xVar,
                    "x_" + xVarName + "SenseMean": xVarMean,
                    "x_" + xVarName + "SenseStd": xVarStd,
                    "y_" + yVarName + "SenseMean": yVarMean,
                    "y_" + yVarName + "SenseStd": yVarStd
                },
                "tags": {
                    "user": getpass.getuser()
                }
            }
            toInfluxDB([point], dbSink)
            # < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < < 

            xVar += step

            # turn everything off (power-cycle the PS)
            ps.turnOff()
            continue
        break

    powerPassiveChannels(config, key, "turnOff")

    with open("output_"+w+".json", "w") as f:
        json.dump(data, f, indent=4)

    


def toInfluxDB(points, dbSink, host='localhost', port=8086):

    host    = dbSink["host"]
    port    = dbSink["port"]
    dbName  = dbSink["database"]
    usr     = dbSink["username"]
    pwd     = ""

    if "INFLUXDBPWD" in  os.environ:
        # Don't ask the user twice (if the pwd was already entered in a previous stage)
        pwd = str(os.environ['INFLUXDBPWD'])

    try:
        # Try to connect without authentication
        client = InfluxDBClient(host=host,
                                port=port,
                                database=dbName,
                                username=usr,
                                password=pwd)

        # Check if it worked
        database_list=client.get_list_database()
        
    except exceptions.InfluxDBClientError as e:

        # If auth is enabled, ask the user for his/her password
        if "authorization failed" in str(e):
            print("[ warn ] Input the password for user \"{}\" in InfluxDB.\n" 
                  "         If you don\'t want to input the password in the future until\n"
                  "         you close your current bash session use \n"
                  "                  \"export INFLUXDBPWD=yourPassword\".".format(usr))
            try:
                pwd = getpass.getpass()
            except:
                print()
                sys.exit(1)

    # And connect again.
    client=InfluxDBClient(host=host,
                          port=port,
                          database=dbName,
                          username=usr,
                          password=pwd)

    try:
        # This should be working now if the user entered the right pwd
        database_list=client.get_list_database()

    except exceptions.InfluxDBClientError as e:

        # If not, let the user know what's happening and exit.
        print("[ error ] Received error from InfluxDB: {}".format(e))
        print("[ info ] Please specify the db connectivity parameters "
                "\"database\", and also \"username\", " 
                "\"host\" and \"port\" in the .\"influxdb_cfg\" section in" 
                "the provided DCS config file. Make sure the provided username and password are correct.\n")

        sys.exit(1)

    try:
        client.write_points(points)
    except exceptions.InfluxDBClientError as e:
        print("[ error ] Received error from InfluxDB: {}".format(e))
        sys.exit(1)


if __name__ == "__main__":

    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-w",
        "--what",
        required=True,
        help="'iv' for IV curve, 'vi' for VI curve.",
    )
    parser.add_argument(
        "-j",
        "--json",
        required=True,
        help="path to iviscan.json",
    )
    parser.add_argument(
        "-e",
        "--equip",
        required=True,
        help="powersupply.json Configuration file with the power supply definition",
    )

    args = parser.parse_args()

    # - - - - - - - - - -
    with open(args.json) as f:
        config = json.load(f)
    
    with open(args.equip) as g:
        psConfig = json.load(g)

    dbSink = psConfig["influxdb_cfg"]


    hw = labRemote.ec.EquipConf()
    hw.setHardwareConfig(args.equip)

    doivi(config, args.what, dbSink)





